package PureMessage::Reports::Quarantine;

use strict;
use warnings;
use PureMessage::Config qw(%config);
use PureMessage::Quarantine;
use PureMessage::Reports::DB qw(get_dbh);
use PureMessage::Reports::Util qw(round_time_hour get_hostname);
use ActiveState::DateTime qw(gmt_offset);

#
# This module collects and stores the quarantine data used for
# reportsing
#

return 1;

sub new {
    my ($class, %args) = @_;

    my $data = \%args;

    $data->{'dbh'} = get_dbh();
    $data->{'ms'} = PureMessage::Quarantine->new();

    my $self = bless $data, $class;
    $self->_init();
    return $self;
}

sub DESTROY {
    my $self = shift;

    $self->{'dbh'}->disconnect();
}

sub num_msg { return shift->{'num_msg'} }

sub size { return shift->{'size'} }

sub pmdb_num_msg { return shift->{'pmdb_num_msg'} }

sub pmdb_size { return shift->{'pmdb_size'} }

sub cur_time { return shift->{'curr_time'} }

sub started { return shift->{'started'} }

sub _init {
    my $self = shift;

    my %status = %{$self->{'ms'}->status()};
    $self->{'num_msg'} = $status{'num_msg'};
    $self->{'size'} = int(($status{'disk_used'} + 512) / 1024);
    $self->{'pmdb_num_msg'} = $status{'pmdb#num_msg'};
    $self->{'pmdb_size'} = $status{'pmdb#disk_used'};
    $self->{'curr_time'} = time;
}

sub reinit {
    my $self = shift;

    $self->_init();
}

sub write_DB {
    my $self = shift;

    # get the period ID
    my $gmt_offset = gmt_offset();
    $gmt_offset = '+' . $gmt_offset unless $gmt_offset =~ /\A[-+]/;
    my $cur_time;
    my ($sec, $min, $hour, $mday, $mon, $year) = localtime($self->cur_time());
    $cur_time = sprintf("%4.4d", ($year + 1900)) . "-"
        . sprintf("%2.2d", ($mon + 1)) . "-"
        . sprintf("%2.2d", $mday) . "T"
        . sprintf("%2.2d", $hour) . ":"
        . sprintf("%2.2d", $min) . ":"
        . sprintf("%2.2d", $sec)
        . $gmt_offset;
    my $super_cur_time;
    ($sec, $min, $hour, $mday, $mon, $year) = localtime(round_time_hour($self->cur_time()));
    $super_cur_time = sprintf("%4.4d", ($year + 1900)) . "-"
        . sprintf("%2.2d", ($mon + 1)) . "-"
        . sprintf("%2.2d", $mday) . "T"
        . sprintf("%2.2d", $hour) . ":"
        . sprintf("%2.2d", $min) . ":"
        . sprintf("%2.2d", $sec)
        . $gmt_offset;
    my ($period_id) = $self->{'dbh'}->selectrow_array(<<"SQL_STATEMENT", undef,($super_cur_time, $cur_time, $cur_time));
SELECT id FROM prd_period
    WHERE duration = 300
    AND super_period_start = ?::timestamptz
    AND ?::timestamptz >= period_start
    AND ?::timestamptz < period_end
SQL_STATEMENT
;
    return 0 unless $period_id;

    # get the host ID
    my $host = get_hostname($self->{'dbh'});
    
    # no need to update the prd_pmx_host dimension because it's updated by the Utils module
    
    my ($host_id) = $self->{'dbh'}->selectrow_array(<<"SQL_STATEMENT", undef, $host);
SELECT id FROM prd_pmx_host
    WHERE name = ?
SQL_STATEMENT
;
    return 0 unless $host_id;

    # get the runtime fact type IDs
    my $type_id_statement = $self->{'dbh'}->prepare(<<"SQL_STATEMENT");
SELECT id FROM prd_runtime_fact_type
    WHERE name = ?
SQL_STATEMENT
;
    $type_id_statement->execute('total_quarantined_local');
    my ($num_msg_id) = $type_id_statement->fetchrow_array();
    $type_id_statement->execute('quarantine_kbytes_local');
    my ($size_id) = $type_id_statement->fetchrow_array();
    $type_id_statement->execute('total_quarantined_central');
    my ($pmdb_num_msg_id) = $type_id_statement->fetchrow_array();
    $type_id_statement->execute('quarantine_kbytes_central');
    my ($pmdb_size_id) = $type_id_statement->fetchrow_array();
    $type_id_statement->finish();

    # populate prd_pmx_runtime_fact
    my $num_msg = $self->num_msg();
    my $size = $self->size();
    my $update_statement = $self->{'dbh'}->prepare(<<"UPDATE_STATEMENT");
UPDATE prd_pmx_runtime_fact
    SET value_absolute = ?
    WHERE period_id = ?
    AND runtime_fact_type_id = ?
    AND pmx_host_id = ?;
UPDATE_STATEMENT
    my $insert_statement = $self->{'dbh'}->prepare(<<"INSERT_STATEMENT");
INSERT INTO prd_pmx_runtime_fact
    (
     period_id,
     runtime_fact_type_id,
     pmx_host_id,
     value_absolute
     )
    SELECT * FROM (
                   SELECT ?::integer AS period_id,
                   ?::integer AS type_id,
                   ?::integer AS host_id,
                   ?::integer AS val
                   ) AS new_fact
    WHERE NOT EXISTS (
                      SELECT * FROM prd_pmx_runtime_fact AS fact
                      WHERE fact.period_id = new_fact.period_id
                      AND fact.runtime_fact_type_id = new_fact.type_id
                      AND fact.pmx_host_id = new_fact.host_id
                      AND fact.value_absolute = new_fact.val
                      );
INSERT_STATEMENT
;

    my $rv = $update_statement->execute(
                                        $self->num_msg(),
                                        $period_id,
                                        $num_msg_id,
                                        $host_id,
                                        );
    my $trv ||= $rv;  # we want to make sure something actually ran
                      #   so I'm ORing the return values
    $rv = $insert_statement->execute(
                                     $period_id,                                    
                                     $num_msg_id,
                                     $host_id,
                                     $self->num_msg(),
				     );
    $trv ||= $rv;

    $rv = $update_statement->execute(
                                     $self->size(),
                                     $period_id,
                                     $size_id,
                                     $host_id,
                                     );
    $trv ||= $rv;
    $rv = $insert_statement->execute(
                                     $period_id,
                                     $size_id,
                                     $host_id,
                                     $self->size()
				     );
    $trv ||= $rv;
    $rv = $update_statement->execute(
                                     $self->pmdb_num_msg(),
                                     $period_id,
                                     $pmdb_num_msg_id,
                                     $host_id,
                                     );
    $trv ||= $rv;
    $rv = $insert_statement->execute(
                                     $period_id,
                                     $pmdb_num_msg_id,
                                     $host_id,
                                     $self->pmdb_num_msg()
				     );
    $trv ||= $rv;
    $rv = $update_statement->execute(
                                     $self->pmdb_size(),
                                     $period_id,
                                     $pmdb_size_id,
                                     $host_id,
                                     );
    $trv ||= $rv;
    $rv = $insert_statement->execute(
                                     $period_id,
                                     $pmdb_size_id,
                                     $host_id,
                                     $self->pmdb_size()
				     );
    $trv ||= $rv;
    $update_statement->finish();

    $self->{'dbh'}->commit();
    return $trv ? 1 : 0;
}

sub start {
    my $self = shift;

    $self->{started} = 1;

    $self->reinit();
    return 1;
}

sub stop {
    my $self = shift;

    $self->{started} = 0;

    return 1;
}

sub run {
    my $self = shift;

    return $self->write_DB();
}
